# -*- python -*-
# $RCSfile: sampleregclassfactory.py,v $
# $Revision: 1.6 $
# $Author: langer $
# $Date: 2006/06/19 17:25:55 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.common import debug
from oof2.common.IO.GUI import regclassfactory

# A special registered class factory for the sampling widget -- has
# functions allowing callers to specify what sorts of samplings are
# allowed.
class SampleRCF(regclassfactory.RegisteredClassFactory):
    def __init__(self, registry, obj=None, title=None, callback=None,
                 fill=0, expand=0, scope=None, name=None, widgetdict={},
                 *args, **kwargs):
        self.sample_types = []
        self.directness = None
        regclassfactory.RegisteredClassFactory.__init__(
            self, registry, obj, title, callback, fill, expand, scope,
            name, widgetdict, *args, **kwargs)
        
    # The "self.directness" reflects whether or not the output
    # operation requires direct output (i.e. is not statistical),
    # and self.sample_types reflects the available sample types
    # from the domain.
    #   Choose a sampling whose sample type is in the list, and which
    # is direct if the operation is direct.  If the operation
    # is not direct, then the sampling may or may not be direct, it's
    # allowed either way.  (This means you can average over points, also.)
    def includeRegistration(self, registration):
        if registration.sample_type in self.sample_types:
            if registration.direct and self.directness:
                return 1
            if not (registration.direct or self.directness):
                return 1
            return None
        return None

    def setSampleCriteria(self, sample_types, directness):
        self.sample_types = sample_types
        self.directness = directness
