// -*- C++ -*-
// $RCSfile: boundarycond.h,v $
// $Revision: 1.14 $
// $Author: langer $
// $Date: 2006/12/19 20:29:44 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef BOUNDARYCOND_H
#define BOUNDARYCOND_H

#include <Python.h>
#include "common/tostring.h"
#include "sparselink.h"

class EdgeSet;
class Field;
class Flux;
class Equation;
class FuncNode;
class CSubProblem;



// FloatBCApp is the "applicator" for the floating boundary
// condition class, which is in Python.  The calling semantics
// are that, when you get to this level, you should have
// a DOF and a nodalequation in-hand, along with the
// mapping lists. 

// For now, instancing the FloatBCApp sets things up by
// setting firstcall to 1.

class FloatBCApp {
private:
  std::vector<double> profile_vector;      // Profile-function offsets.
  std::vector<int> rhssubtractor_index;     // Column-number map
public:
  FloatBCApp() {}
  ~FloatBCApp() {}
  int getdofindex(CSubProblem*, FuncNode*, Field*, int);
  int geteqnindex(CSubProblem*, FuncNode*, Equation*, int);
  void editmap(CSubProblem *, double, FuncNode *, Field *, int, Equation *, 
	       int, int, int, double);
  void profile_rhs(CSubProblem *);
};

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

// "Applicator" for the flux boundary condition.  

class NeumannBCApp {
private:
  CSubProblem *subproblem;
  Flux *flux;
  EdgeSet *bdy;
public:
  NeumannBCApp(CSubProblem *m, Flux *f, EdgeSet *b)
    : subproblem(m), flux(f), bdy(b)
  {}
  ~NeumannBCApp() {}
  void integrate(PyObject *, PyObject *, bool normal, double time);
};


// ForceBC is the node-based twin of NeumannBC.  Flux's make sense to
// integrate, forces are associated with equations and make sense for
// direct application at nodes.

void applyForceBC(CSubProblem*, Equation*, FuncNode*, int eqnindex,
		  double value);
  
#endif

