# -*- python -*-
# $RCSfile: materialplugin.py,v $
# $Revision: 1.6 $
# $Author: langer $
# $Date: 2006/12/19 20:29:44 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 



## Define a MicrostructurePlugIn to catch "material changed"
## switchboard signals and tell the relevant subproblems of the event.
from oof2.SWIG.common import switchboard
from oof2.SWIG.engine import material
from oof2.common import debug
from oof2.common import microstructure
from oof2.engine import materialmanager
from oof2.engine import subproblemcontext
from oof2.engine import conjugate

class MaterialMSPlugIn(microstructure.MicrostructurePlugIn):
    def __init__(self, ms):
        microstructure.MicrostructurePlugIn.__init__(self, ms)
        self.sbcallback = switchboard.requestCallback("material changed",
                                                      self.matChanged)
    def destroy(self):
        switchboard.removeCallback(self.sbcallback)
        microstructure.MicrostructurePlugIn.destroy(self)
    def matChanged(self, materialname):
        # The given material has changed.  Look through the materials
        # in our Microstructure to see if we care.
        matl = materialmanager.getMaterial(materialname)
        if matl in material.getMaterials(self.microstructure):
            # We *do* care. Really.
            # Tell all the SubProblems belonging to the microstructure that
            # the material has changed.
            msname = self.microstructure.name()
            subppaths = subproblemcontext.subproblems.keys(base=msname)
            for subppath in subppaths:
                subproblemcontext.subproblems[[msname]+subppath].changed()
                conjugate.check_symmetry([msname]+subppath)
                

microstructure.registerMicrostructurePlugIn(MaterialMSPlugIn)
