// -*- C++ -*-
// $RCSfile: nonlinear.C,v $
// $Revision: 1.7 $
// $Author: langer $
// $Date: 2006/12/19 20:29:47 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include "engine/elementnodeiterator.h"
#include "engine/femesh.h"
#include "engine/field.h"
#include "engine/flux.h"
#include "engine/material.h"
#include "engine/property/elasticity/nonlinear/nonlinear.h"
#include "engine/property/orientation/orientation.h"
#include "engine/element.h"
#include "engine/matvec.h"
#include "engine/symmmatrix.h"

NonLinearElasticityProp::NonLinearElasticityProp(PyObject *registry,
						 const std::string &nm, 
						 Cijkl *cijkl,
						 double hardening)
  : Elasticity(nm, registry),
    cijkl_(*cijkl),
    hardening(hardening)
{ }


// void NonLinearElasticityProp::cross_reference(Material *mat) {
//   Trace("AnisoElasticity::cross_reference");
//   orientation = dynamic_cast<OrientationProp*>(mat->fetchProperty("Orientation"));
// }

const Cijkl NonLinearElasticityProp::cijkl(const FEMesh *mesh,
					   const Element *element, 
					   const MasterPosition &x) const
{
  Cijkl modulus = cijkl_;

  static CompoundField *displacement_ = dynamic_cast<CompoundField*>
    (Field::getField("Displacement"));
  Field *displacement_z = displacement_->out_of_plane();
  bool inplane = displacement_->in_plane(mesh);

  // Get local strain
  SymmMatrix strain(3);
  for(ElementFuncNodeIterator nu=element->funcnode_iterator(); !nu.end(); ++nu){
    if(nu.hasField(*displacement_)) {
      // Shape function derivatives at postion pos.
      double dsfx = nu.dshapefunction(0, x);
      double dsfy = nu.dshapefunction(1, x);
      // Displacement components at node nu.
      double dispx = (*displacement_)(nu, 0)->value();
      double dispy = (*displacement_)(nu, 1)->value();
      strain(0,0) += dispx*dsfx;
      strain(1,1) += dispy*dsfy;
      strain(0,1) += 0.5*(dispx*dsfy + dispy*dsfx);

      if(!inplane) {
	double sf = nu.shapefunction(x);
	strain(0,2) += sf*(*displacement_z)(nu, 0)->value();
	strain(1,2) += sf*(*displacement_z)(nu, 1)->value();
	strain(2,2) += sf*(*displacement_z)(nu, 2)->value();
      }
    }
  }
  //
  modulus(0,0) *= 1.0 - hardening*strain(0,0);
  modulus(1,1) *= 1.0 - hardening*strain(1,1);
  if(!displacement_->in_plane(mesh))
    modulus(2,2) *= 1.0 - hardening*strain(2,2);
  // 
  return modulus;
}
