// -*- C++ -*-
// $RCSfile: orientation.h,v $
// $Revision: 1.15 $
// $Author: langer $
// $Date: 2007/03/02 21:31:48 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef ORIENTATION_H
#define ORIENTATION_H

#include "engine/property.h"
#include "engine/eulerangle.h"
#include "engine/matvec.h"
#include <string>

class SymmMatrix3;

class OrientationPropBase : public Property {
public:
  OrientationPropBase(PyObject *registry, const std::string &name)
    : Property(name, registry)
  {}
  virtual const EulerAngle &eulerangle() const = 0;
  virtual bool constant_in_space() const = 0;


};

class OrientationProp : public OrientationPropBase {
private:
  EulerAngle orient;
public:
  OrientationProp(PyObject *registry, const std::string &name, 
		  const EulerAngle &ea);
  OrientationProp(PyObject *registry, const std::string &name,
		  EulerAngle *ea);
  const EulerAngle &eulerangle() const { return orient; }
  EulerAngle eulerangle(const FEMesh*, const Element*, const MasterPosition&)
    const;
  virtual bool constant_in_space() const { return true; }

  // From the comments in eulerangle.C:
  // ... The rotation matrix multiplied by a vector gives the
  // coordinates of the vector in a coordinate system that has been
  // rotated by the Euler angle.

//   MATRIX_D rotate(const MATRIX_D&) const;
//   VECTOR_D rotate(const VECTOR_D&) const;
//   SymmMatrix3 rotate(const SymmMatrix3&) const;
  virtual bool is_symmetric(const CSubProblem*) const { return true; }
};

#endif
