// -*- C++ -*-
// $RCSfile: permittivity.h,v $
// $Revision: 1.15 $
// $Author: langer $
// $Date: 2007/03/02 21:31:48 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

// dielectric permittivity property

#ifndef PERMITTIVITY_H
#define PERMITTIVITY_H


#include "engine/property.h"
#include "engine/matvec.h"
#include "engine/symmmatrix.h"
#include "common/pythonexportable.h"
#include <string>

class CSubProblem;
class Element;
class ElementNodeIterator;
class Material;
class OrientationProp;
class OutputVal;
class PropertyOutput;
class ScalarField;
class VectorFlux;

class DielectricPermittivity : public Property {
public:
  DielectricPermittivity(PyObject *registry, const std::string &name);
  virtual void fluxmatrix(const FEMesh*, const Element*, 
			  const ElementFuncNodeIterator&,
			  const Flux*, FluxData*,
			  const MasterPosition&) const;

  virtual void cross_reference(Material*) = 0;
  virtual void output(const FEMesh*, const Element*, const PropertyOutput*,
		      const MasterPosition&, OutputVal*) const;
  virtual const SymmMatrix3 &muij(const FEMesh *mesh, const Element *element,
				 const MasterPosition&x) const = 0;
  virtual int integration_order(const CSubProblem*, const Element*) const;
  virtual bool is_symmetric(const CSubProblem*) const { return true; }
  virtual bool constant_in_space() const { return true; }
protected:
  SymmMatrix3 permittivitytensor;
  ScalarField *voltage;
  VectorFlux *total_polarization;
};

class IsoDielectricPermittivity : public DielectricPermittivity {
public:
  IsoDielectricPermittivity(PyObject *registry, 
		      const std::string &name, double epsilon);
  virtual void cross_reference(Material*) {}
  virtual void precompute();
  virtual const SymmMatrix3 &muij(const FEMesh *mesh, const Element *element,
				  const MasterPosition&x) const
  {
    return permittivitytensor;
  }
private:
  double epsilon_;
};

class AnisoDielectricPermittivity : public DielectricPermittivity {
public:
  AnisoDielectricPermittivity(PyObject *registry,
			    const std::string &name,
			    SymmMatrix3 *epsilon);
  virtual void cross_reference(Material*); // finds Orientation
  virtual void precompute();
  virtual const SymmMatrix3 &muij(const FEMesh *mesh, const Element *element,
				  const MasterPosition&x) const
  { 
    return permittivitytensor;
  }
private:
  SymmMatrix3 epsilon_;
  OrientationProp *orientation;
};

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

// distributed heat source

class ChargeDensity : public Property {
private:
  double qdot_; // external heat dumped in per unit area per unit time.
  VectorFlux *total_polarization;
public:
  ChargeDensity(PyObject *registry, const std::string &name, double qd);
  double rho() { return qdot_; }
  virtual int integration_order(const CSubProblem*, const Element*) const;
  virtual void fluxrhs(const FEMesh*, const Element*, 
		       const Flux*, FluxData*, const MasterPosition&) const;
   virtual bool is_symmetric(const CSubProblem* mesh) const { return true; }
  virtual bool constant_in_space() const { return true; }
};

#endif
