// -*- C++ -*-
// $RCSfile: thermalexpansion.h,v $
// $Revision: 1.25 $
// $Author: langer $
// $Date: 2007/02/28 21:06:44 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#ifndef THERMALEXPANSION_H
#define THERMALEXPANSION_H

// Base class for Thermal Expansion Properties

#include "engine/property.h"
#include "engine/matvec.h"
#include "engine/property.h"
#include "engine/symmmatrix.h"
#include <string>

class Elasticity;
class Element;
class FEMesh;
class Flux;
class FluxData;
class Material;
class Position;
class OrientationProp;
class OutputVal;
class PropertyOutput;
class ScalarField;
class SymmetricTensorFlux;

class ThermalExpansion : public Property {
protected:
  // Physical parameters.
  double tzero_;
  Elasticity *elasticity;	// pointer to this Material's Elasticity
  ScalarField *temperature;
  SymmetricTensorFlux *stress_flux;
  SymmMatrix3 expansiontensor; //lab reference system tensor
public:
  ThermalExpansion(PyObject *registry, 
		   const std::string &nm, 
		   double t0);
  virtual ~ThermalExpansion() {}
  virtual void cross_reference(Material*) = 0;
  virtual void precompute();
  virtual bool constant_in_space() const { return true; }

  virtual void fluxmatrix(const FEMesh*, const Element*, 
			  const ElementFuncNodeIterator&,
			  const Flux*, FluxData*, const MasterPosition&) const;
  virtual void fluxrhs(const FEMesh*, const Element*,
		       const Flux*, FluxData*, const MasterPosition&) const;
  virtual void output(const FEMesh*, const Element*, const PropertyOutput*,
		      const MasterPosition&, OutputVal*) const;

  virtual int integration_order(const CSubProblem*, const Element*) const;

  virtual bool is_symmetric(const CSubProblem*) const;

};

class IsotropicThermalExpansion: public ThermalExpansion {
public:
  IsotropicThermalExpansion(PyObject *registry, 
			    const std::string &name, 
			    double alpha,
			    double t0);
  virtual void cross_reference(Material*);
  virtual void precompute();
private:
  double alpha_;
};

class AnisotropicThermalExpansion: public ThermalExpansion {
public:
  AnisotropicThermalExpansion(PyObject *registry, 
			      const std::string &nm, 
			      SymmMatrix3 *alpha, double t0);
  virtual void cross_reference(Material*); // finds Orientation
  virtual void precompute();
private:
  SymmMatrix3 alpha_;
  OrientationProp *orientation;
};

#endif
