// -*- C++ -*-
// $RCSfile: rank3tensor.h,v $
// $Revision: 1.7 $
// $Author: reida $
// $Date: 2006/07/19 20:46:59 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

class Rank3Tensor;

#ifndef RANK3TENSOR_H
#define RANK3TENSOR_H

#include <iostream>
#include "engine/eigenvalues.h"
#include "engine/matvec.h"
#include "engine/fieldindex.h"
#include "engine/outputval.h"
#include "common/pythonexportable.h"
#include "engine/symmmatrix.h"


class Cijkl; //actually, a rank 4 tensor

class Rank3Tensor : public PythonExportable<Rank3Tensor> {
private:
  // a rank 3 tensor can be thought of
  // as a group of three symmetric matrices
  // with a special set of operations
  SymmMatrix  m0;
  SymmMatrix  m1;
  SymmMatrix  m2;
  static std::string classname_;
  static std::string modulename_;

protected:
  unsigned int nrows;

public:
  Rank3Tensor() :  m0(3), m1(3), m2(3), nrows(3) {} //ok
  Rank3Tensor(const Rank3Tensor&); //ok
  virtual ~Rank3Tensor();//ok
  Rank3Tensor &operator=(const Rank3Tensor&);//ok
  Rank3Tensor &operator*=(double);//ok
  Rank3Tensor &operator/=(double);//ok
  double &operator()(int i, int j, int k); //ok
  double operator()(int i, int j, int k) const;//ok
  Rank3Tensor &operator+=(const Rank3Tensor&);//ok
  Rank3Tensor &operator-=(const Rank3Tensor&);//ok
  bool operator==(const Rank3Tensor&);
  int badindex(unsigned int i) const { return i < 0 || i >= nrows; } //ok
	
  Rank3Tensor transform(const MATRIX_D&) const; // A^T B A //ok
	
  friend class Debug;
  friend std::ostream& operator<<(std::ostream&, const Rank3Tensor&); //ok
  friend Rank3Tensor operator*(double, const Rank3Tensor&); //ok
  friend Rank3Tensor operator*(const Rank3Tensor&, double); //ok
  friend SymmMatrix operator*(const Rank3Tensor&, const VECTOR_D&);//ok
  virtual const std::string &classname() const { return classname_; }  //ok
  virtual const std::string &modulename() const { return modulename_; } //ok
  friend Rank3Tensor operator/(const Rank3Tensor &A, double x); //ok
  double &operator()(const int &, const SymTensorIndex&);
  double operator()(const int &, const SymTensorIndex&) const;
  int size() {return nrows;}
  //from oof1
  SymmMatrix &operator()(const int i);//ok
  SymmMatrix operator()(const int i) const;//ok
  Rank3Tensor operator*(const Cijkl&); //ok
  SymmMatrix operator*(const VECTOR_D&); //ok 
  const Rank3Tensor operator +(const Rank3Tensor & A); //ok
  friend Rank3Tensor operator*(const Cijkl&, const VECTOR_D&);//for electrostriction //ok
  friend Rank3Tensor operator*(const Cijkl&, const Rank3Tensor&); //ok

};

#endif
