# -*- python -*-
# $RCSfile: file_compare.py,v $
# $Revision: 1.1 $
# $Author: langer $
# $Date: 2007/01/23 14:39:32 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import re
import sys

# Flag that says whether to generate missing reference data files.
# Should be false unless you really know what you're doing.
generate=False

def fp_file_compare(file1, file2, tolerance, comment="#", pdfmode=False):
    # Regexp for matching floating-point numbers, copied from section
    # 4.2.6 of the Python 2.3 documentation.  The "(...)" group
    # constructs in the original have been replaced by "(?:...)"
    # constructs, as a way of grouping sub-expressions without
    # creating explicit groups in the regexp itself. The explicit
    # groups cause split and match to be annoying.
    floatpattern = re.compile(
        "[-+]?(?:\d+(?:\.\d*)?|\d*\.\d+)(?:[eE][-+]?\d+)?")

    # Pattern for detecting PDF date strings, which should not be
    # compared.  This looks for a non-digit or beginning of a line,
    # followed by exactly 14 digits, followed by 'Z'.
    datepattern = re.compile("(?:\D|^)\d{14}Z")

    try:
        f2 = file(file2)
    except:
        if generate:
            os.rename(file1,file2)
            print >> sys.stderr, "\nMoving file %s to %s.\n" % (file1, file2)
            return True
        else:
            raise

    f1 = file(file1)

    try:
        for f1_line in f1:
            if f1_line[0]==comment:
                f2_line = f2.next()
                continue
            f2_line = f2.next()

            # If we're comparing PDF files, and both lines contain
            # date strings, just go on to the next lines.  This
            # ignores anything else on a line containing dates.  It's
            # not generally true that there's nothing else interesting
            # on date lines, but it is true of the pdf's generated by
            # oof.
            if pdfmode and datepattern.search(f1_line) and \
                   datepattern.search(f2_line):
                continue
                
            f1_text_items = floatpattern.split(f1_line)
            f2_text_items = floatpattern.split(f2_line)
            f1_float_items = floatpattern.findall(f1_line)
            f2_float_items = floatpattern.findall(f2_line)

            for (item1, item2) in zip(f1_text_items, f2_text_items):
                if item1 != item2:
                    print >> sys.stderr, \
                          "Text mismatch, %s != %s." % (item1, item2), \
                          "in", file1, file2
                    return False
            
            for(item1, item2) in zip(f1_float_items, f2_float_items):
                try:
                    int1 = int(item1)
                    int2 = int(item2)
                except ValueError:
                    try:
                        float1 = float(item1)
                        float2 = float(item2)
                    except ValueError:
                        print >> sys.stderr, \
                              "Numeric conversion problem for item %s or %s" %\
                              (item1, item2), "in", file1, file2
                    else:
                        diff = abs(float1 - float2)
                        reltol = min(abs(float1), abs(float2))*tolerance
                        # This uses the same tolerance for both absolute
                        # and relative error, which isn't usually a good
                        # idea, but is ok if the numbers being compared
                        # are more or less of order 1.
                        ok = diff < reltol or diff < tolerance
                        if not ok:
                            print >> sys.stderr, \
                                  "Failed comparison: %g != %g\n" % \
                                   (float1, float2), "in", file1, file2
                            return ok # Certainly false.
                else: # Integer conversion worked, do comparison.
                    if int1!=int2:
                        print >> sys.stderr, \
                              "Integer comparison failed, %d != %d." % \
                              (int1, int2), "in", file1, file2
                        return False
        return True
    finally:
        f1.close()
        f2.close()
    
